@@ -21,8 +21,9 @@ import typing as T
 
 
 isAsyncGenerator=lambda x: "async_generator" in str(type(x)) #TODO: howto better ?
+fixBacktip=lambda x: x.replace("`",r"\`")
 
-value=lambda x: x.get() if isinstance(x,ReactiveProp) else x
+value=lambda x: x.getValue() if isinstance(x,ReactiveProp) else x
 
 def jjs(obj): #TODO: not optimal ... do better than str's pattern subst ;-)
     """ json dumps (js is b'' (bytes)) """
@@ -77,93 +78,157 @@ class Tag(metaclass=MyMetaclass):
         return """<%(tag)s%(attrs)s>%(content)s</%(tag)s>""" % dict(
             tag=self.tag.replace("_","-"),
             attrs=" ".join([""]+rattrs) if rattrs else "",
-            content=" ".join([str(i) for i in self.__contents if i is not None]),
+            content=" ".join([str(i) for i in self.__contents if i is not None and not isinstance(i,bool)]),
         )
     def __repr__(self):
         return "<%s>" % self.__class__.__name__
 
-class CSS(Tag):
-    def __init__(self,content):
-        self._md5=hashlib.md5("css:".encode()+content.encode()).hexdigest()
-        if content.startswith("http"):
-            self.tag="link"
-            super().__init__(type="text/css",rel="stylesheet",href=content)
-        else:
-            self.tag="style"
-            super().__init__(content,type="text/css")
-
-class JS(Tag):
-    tag="script"
-    def __init__(self,content):
-        self._md5=hashlib.md5("js:".encode()+content.encode()).hexdigest()
-        if content.startswith("http"):
-            super().__init__(type="text/javascript",src=content)
-        else:
-            super().__init__(content,type="text/javascript")
-
+class NONE: pass
 class ReactiveProp:
-    def __init__(self,dico:dict,attribut:str):
-        self.__instance=dico
-        self.__attribut=attribut
-    def set(self,v):
-        self.__instance[self.__attribut]=v
-    def get(self):
-        return self.__instance[self.__attribut]
+    def __init__(self,dico:dict,attribut:str,value=NONE):
+        assert not isinstance(dico,ReactiveProp)
+        assert not isinstance(attribut,ReactiveProp)
+        assert not isinstance(value,ReactiveProp)
+        self._instance=dico
+        self._attribut=attribut
+        if value!=NONE:
+            self.setValue(value)
+    def setValue(self,v):
+        assert not isinstance(v,ReactiveProp)
+        self._instance[self._attribut]=v
+    def getValue(self):
+        return self._instance[self._attribut]
 
 
     def __eq__(self, v):
-        return self.get() == value(v)
+        return self.getValue() == value(v)
 
     def __ne__(self, v):
-        return self.get() != value(v)
+        return self.getValue() != value(v)
 
     def __lt__(self, v):
-        return self.get() < value(v)
+        return self.getValue() < value(v)
 
     def __le__(self, v):
-        return self.get() <= value(v)
+        return self.getValue() <= value(v)
 
     def __ge__(self, v):
-        return self.get() >= value(v)
+        return self.getValue() >= value(v)
 
     def __gt__(self, v):
-        return self.get() > value(v)
+        return self.getValue() > value(v)
 
+    def __add__(self,v):
+        return self.getValue() + value(v)
+    def __sub__(self,v):
+        return self.getValue() - value(v)
+    def __mul__(self,v):
+        return self.getValue() * value(v)
+    def __truediv__(self,v):
+        return self.getValue() / value(v)
+    def __floordiv__(self,v):
+        return self.getValue() // value(v)
+    def __mod__(self,v):
+        return self.getValue() % value(v)
 
     def __iadd__(self,v):
-        vv=self.get() + value(v)
-        self.set( vv )
-        return self
+        return self.getValue() + value(v)
+    def __isub__(self,v):
+        return self.getValue() - value(v)
+    def __imul__(self,v):
+        return self.getValue() * value(v)
+    def __itruediv__(self,v):
+        return self.getValue() / value(v)
+    def __ifloordiv__(self,v):
+        return self.getValue() // value(v)
+
+
+    def __radd__(self, v):
+        return value(v) + self.getValue()
+    def __rsub__(self, v):
+        return value(v) - self.getValue()
+    def __rmul__(self, v):
+        return value(v) * self.getValue()
+    def __rtruediv__(self, v):
+        return value(v) / self.getValue()
+    def __rfloordiv__(self, v):
+        return value(v) // self.getValue()
+    def __rmod__(self,v):
+        return value(v) % self.getValue()
+
+    def __and__(self,v):
+        return self.getValue() & value(v)
+    def __or__(self,v):
+        return self.getValue() | value(v)
+    def __xor__(self,v):
+        return self.getValue() ^ value(v)
+
+    def __rand__(self,v):
+        return value(v) & self.getValue()
+    def __ror__(self,v):
+        return value(v) | self.getValue()
+    def __rxor__(self,v):
+        return value(v) ^ self.getValue()
+
 
     def __getitem__(self,k):
-        return self.get()[k]
+        return self.getValue()[k]
     def __setitem__(self,k,v):
-        self.get()[k]=value(v)
+        self.getValue()[k]=value(v)
+    def __delitem__(self,k):
+        del self.getValue()[ value(k) ]
 
+    def __float__(self):
+        return float(self.getValue())
     def __int__(self):
-        return int(self.get())
+        return int(self.getValue())
     def __bool__(self):
-        return bool(self.get())
+        return bool(self.getValue())
     def __str__(self):
-        return str(self.get())
+        return str(self.getValue())
 
     def __repr__(self):
-        iid=self.__instance.id if hasattr(self.__instance,"id") else str(self.__instance)
-        return "<%s instance=%s attr=%s>" % (self.__class__.__name__,iid,self.__attribut)
+        iid=self._instance.id if hasattr(self._instance,"id") else str(self._instance)
+        return "<ReactiveProp:%s attr=%s of instance=%s>" % (self.__class__.__name__,self._attribut,iid)
+
+
+    @property
+    def __class__(self):
+        return type(self.getValue())
+
+    def __hash__(self):
+        return hash(self.getValue())
+
+    def __iter__(self):
+        return iter(self.getValue())
+
+    def __next__(self):
+        return next(self.getValue())
+
+    def __contains__(self,x):
+        return value(x) in self.getValue()
+
+
 
     def __getattr__(self,k):
-        return getattr(self.get(),k)
+        return getattr(self.getValue(),k)
     def __setattr__(self,k,v):
         if k.startswith("_"):
             super().__setattr__(k, v)
         else:
-            setattr(self.get(),k,value(v))
+            setattr(self.getValue(),k,value(v))
 
+    def __len__(self):
+        return len(self.getValue())
+
+    def __call__(self,*a,**k):
+        return self.getValue()(*a,**k)
     #TODO: add a lot of __slot__ ;-)
 
 
 class render:
 
+    # POST build
     @staticmethod
     def local( method ): # gtag.event decorator
         """ Make the method renders only this component (and its childs)"""
@@ -177,11 +242,12 @@ class render:
         return method
 
     @staticmethod
-    def no( method ): # gtag.event decorator
+    def none( method ): # gtag.event decorator
         """ Make the method renders nothing """
         Capacity(method).set(inspect.getouterframes(inspect.currentframe())[0].function)
         return method
 
+
 class Capacity:
     def __init__(self,method:callable):
         self.__method=method
@@ -194,30 +260,29 @@ class Capacity:
         self.__method.capacities.append(capacity)
 
 
-
-
-class GtagProxy:
-    """ Expose props(as ReactiveProps)/method from a gtag """
+class Binder:
     def __init__(self,instance):
         self.__instance=instance
     def __getattr__(self,name:str):
-        if name in ["id","main","parent"]:
-            return getattr(self.__instance,name)
-        elif name in self.__instance.__dict__.keys(): # bind a data attribut  -> return a ReactiveProp
-            o=self.__instance.__dict__[name]
-            if isinstance(o,ReactiveProp):
-                return o
-            else:
-                return ReactiveProp(self.__instance.__dict__,name)
-        elif name in dir(self.__instance):   # bind a self.method    -> return a js/string for a guy's call in js side
-            def _(*a,**k):
-                method=getattr(self.__instance,name)
-                return method(*a,**k)
+        m=hasattr(self.__instance,name) and getattr(self.__instance,name)
+        if m and callable( m ):   # bind a self.method    -> return a js/string for a guy's call in js side
+
+            #/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\
+            # resolve all declaredJsVars from all childs
+            pool={}
+            for id,o in self.__instance._getChilds().items():
+                pool.update( {o.__class__.__name__:o._declaredJsInputs} )
+            #/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\
+
+            def _(*args,**kargs):
+                if args or kargs:
+                    args=[value(i) for i in args]
+                    kargs={k:value(v) for k,v in kargs.items()}
+
+                return "self.bindUpdate('%s',GID,'%s',%s,%s,%s)" % (self.__instance.id,name,jjs(args),jjs(kargs),jjs(pool))
             return _
         else:
-            raise Exception("Unknown method/attribut '%s' in '%s'"%(name,repr(self.__instance)))
-    def __repr__(self):
-        return repr(self.__instance)
+            raise Exception("Unknown method '%s' in '%s'"%(name,self.__instance.__class__.__name__))
 
 
 class GTag:
@@ -226,11 +291,15 @@ class GTag:
     """
     size=None
     _call=None # first event to call at start !
+    _parent=None
+
     """ size of the windowed runned gtag (tuple (width,height) or guy.FULLSCREEN or None) """
 
     # implicit parent version (don't need to pass self(=parent) when creating a gtag)
     def __init__(self,*a,**k):
-        self._tag=None
+        self._data={}
+        self._id="%s_%s" % (self.__class__.__name__,hex(id(self))[2:])
+        self._tag=NONE
         self._scripts=[]
 
         if "parent" in k.keys(): # clonage (only main tags, so parent is None)
@@ -253,19 +322,23 @@ class GTag:
                     if isinstance(parent,GTag):
                         break
 
-        self._id="%s_%s" % (self.__class__.__name__,hex(id(self))[2:])
         self._childs=[]     #<- this is cleared at each rendering
         self._args=a
         self._kargs=k
         self._parent=parent
 
         log("INIT",repr(self))
+
+        #/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\
+        signature = inspect.signature( self.init )
+        self._declaredJsInputs={k: v.default for k, v in signature.parameters.items() if type(v.default)==bytes}
+        #/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\
+
         self.init(*self._args,**self._kargs)
         self._childs=[]     #<- clear innerchilds (creating during child phase), to avoid to appears in child
 
         self._scriptsInInit=self._scripts[:]
-
-        self._tag = self.build()
+        # self._tag = self.build()    #TODO: remove this build !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 
         # Store the instance in the parent._childs
         if self._parent:
@@ -285,7 +358,8 @@ class GTag:
 
     @property
     def _ichilds(self):
-        return [v for k,v in self.__dict__.items() if k not in ["_tag","_parent"] and isinstance(v,GTag)]
+        ll= [v for k,v in self._data.items() if isinstance(v,GTag)]
+        return ll
 
     def _getChilds(self) -> dict:
 
@@ -306,11 +380,6 @@ class GTag:
         except KeyError:
             raise Exception("ERROR: Unknown child '%s' in '%s'"%(id,self.id))
 
-    def _getMain(self):
-        x=self
-        while x._parent is not None:
-            x=x._parent
-        return x
 
     @property
     def id(self):
@@ -318,47 +387,33 @@ class GTag:
 
 
     @property
-    def parent(self)-> T.Union[GtagProxy,None]:
-        """ return caller/binder to parent instance (None if gtag is the main) """
+    def parent(self): # -> T.Union[GTag,None]:
+        """ return the parent instance (None if gtag is the main) """
         if self._parent is None:
             return None
         else:
-            return GtagProxy( self._parent )
+            return self._parent
 
 
     @property
-    def main(self)-> GtagProxy:
-        """ return caller/binder to main instance """
-        return GtagProxy( self._getMain() )
+    def main(self): # -> GTag:
+        """ return the main instance """
+        x=self
+        while x._parent is not None:
+            x=x._parent
+        return x
 
 
 
     @property
     def bind(self):
-        """ to bind attribute or method !"""
-        class Binder:
-            def __getattr__(this,name:str):
-                if name in self.__dict__.keys(): # bind a data attribut  -> return a ReactiveProp
-                    o=self.__dict__[name]
-                    if isinstance(o,ReactiveProp):
-                        return o
-                    else:
-                        return ReactiveProp(self.__dict__,name)
-                elif name in dir(self):   # bind a self.method    -> return a js/string for a guy's call in js side
-                    def _(*args,**kargs):
-                        if args or kargs:
-                            return "self.bindUpdate('%s',GID,'%s',%s,%s)" % (self.id,name,jjs(args),jjs(kargs))
-                        else:
-                            return "self.bindUpdate('%s',GID,'%s',[],{})" % (self.id,name)
-                    return _
-                else:
-                    raise Exception("Unknown method/attribut '%s' in '%s'"%(name,self.__class__.__name__))
-        return Binder()
+        """ to bind method ! and return its js repr"""
+        return Binder(self)
 
 
-    def _clone(self): #TODO: not clear here ... need redone (the rebuild() needed ?! why ?!)
+    def _clone(self): #TODO: not clear here ... need redone (the rebuild() needed ?! why ?! (because storage in "_tag")
         assert self._parent==None,"Can't clone a gtag which is not the main one"
-        props={k:v for k,v in self.__dict__.items() if k[0]!="_" or k=="_call"}
+        props={k:v for k,v in self.__dict__.items() if k[0]!="_" or k=="_call" or k=="_data"}
         gtag = self.__class__(*self._args,**self._kargs,parent=None) # parent=None, will avoid guess parent ! (it makes sense, because you can clone only mains)
         gtag.__dict__.update(props)
         gtag._scripts=[]
@@ -375,20 +430,17 @@ class GTag:
 
         mklist=lambda x: x if isinstance(x,list) else [x]
 
+        md5= lambda x: hashlib.md5(x.encode('utf-8')).hexdigest()
+
+        lmd5=[]
         ll=[]
         for g in GTag.__subclasses__():
-            if hasattr(g,"css"):
-                for i in mklist(getattr(g,"css")):
-                    if i:
-                        c=CSS(i)
-                        if c._md5 not in [l._md5 for l in ll]:
-                            ll.append( c )
-            if hasattr(g,"js"):
-                for i in mklist(getattr(g,"js")):
-                    if i:
-                        c=JS(i)
-                        if c._md5 not in [l._md5 for l in ll]:
-                            ll.append( c )
+            if hasattr(g,"headers"):
+                for i in mklist(getattr(g,"headers")):
+                    m5=md5(str(i))
+                    if m5 not in lmd5:
+                        lmd5.append(m5)
+                        ll.append( i )
         return ll
 
 
@@ -400,7 +452,20 @@ class GTag:
         """ Override for static build
             SHOULD RETURN a "Tag" (not a GTag)
         """
-        pass
+        raise Exception("Should be implemented")
+
+    async def _start(self):
+        #=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- call statement (old autostart system)
+        if self._call:  # there is an event to call at start !
+            if asyncio.iscoroutine(self._call):
+                await self._call
+            elif isAsyncGenerator(self._call):
+                async for _ in self._call:
+                    assert _ is None, "wtf?"
+                    yield
+            else:
+                raise Exception("Not implemented (calling a sync/start function)")
+        #=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
 
     def exit(self,v=None): pass # overriden by run/runcef/serve
 
@@ -410,44 +475,69 @@ class GTag:
     def _rebuild(self,clearScripts=True):
         if clearScripts: self._clearScripts()
         self._childs=[]
-        self._tag=self.build()
+        self._tag=self.build( )
 
     def __str__(self):
-        def getTagIded(o):
-            if isinstance(o,Tag):
-                o.id=self.id
-            elif isinstance(o,GTag):
-                o=getTagIded(o._tag)
-                o.id=self.id
-            else:
-                o=Tag.span(o) # auto span'ify
-                o.id=self.id
-            return o
-
         log("___rendering",repr(self))
+
+        if self._tag == NONE:
+            self._tag=self.build()
+
         o= self._tag
+
         if o is None:
             return ""
         else:
             for i in self._ichilds:
                 i._rebuild()
 
-            o=getTagIded(o)
+            if isinstance(o,Tag):
+                o.id=self.id
+            elif isinstance(o,GTag):
+                o=Tag.div(o) # auto div'ify
+                o.id=self.id
+            else:
+                o=Tag.span(o) # auto span'ify
+                o.id=self.id
             return str(o)
 
     def __repr__(self):
         return "<GTag: %s [parent:%s] (innerchilds=%s)>" % (
-            self.id,
+            self._id,
             self._parent.id if self._parent else "None",
-            [i.id for i in self._ichilds]
+            [i._id for i in self._ichilds]
         )
 
     def __setattr__(self,k,v):
-        o=self.__dict__.get(k)
-        if isinstance(o,ReactiveProp):
-            o.set( value(v) )
-        else:
+        if k.startswith("_"):
+            # print("REAL SET",k,repr(v))
             super().__setattr__(k,v)
+        else:
+            o=self._data.get(k)
+
+            if isinstance(o,ReactiveProp):
+                # print("SET EXISTING REACTIVE",k,repr(v))
+
+                if isinstance(v,ReactiveProp):
+                    # v is RP
+                    self._data[k]=v
+                    super().__setattr__(k,v)
+                else:
+                    # v is real
+                    o.setValue( v )
+
+            else:
+                # print("CREATE REACTIVE",k,repr(v))
+
+                if isinstance(v,ReactiveProp):
+                    # v is RP
+                    self._data[k]=v            #(put RP in RP !!!!!!!!!!!!!!!!!!!!!!)
+                    super().__setattr__(k,v)
+                else:
+                    # v is real
+                    self._data[k]=v
+                    super().__setattr__(k,ReactiveProp(self._data,k,v))
+
 
     @property
     def scripts(self):
@@ -470,9 +560,14 @@ class GTag:
         s=self._getScripts()
         log(">>>UPDATE:",repr(self))
         log(self._tree())
-        return dict(script="""document.querySelector("#%s").outerHTML=`%s`;%s""" % (
-            self.id, h,s
-        ))
+
+        return dict( script="""
+(function (id,content) {
+    let o=document.querySelector(id);
+    if(o) o.outerHTML=content;
+    else document.body.innerHTML=content;
+})("#%s",`%s`); %s
+        """ % (self.id, fixBacktip(h),s))
 
     def run(self,*a,start=None,**k) -> any:
         """ Run as Guy App (using Chrome) """
@@ -498,7 +593,7 @@ class GTag:
 
 
 class GTagApp(guy.Guy):
-    """ The main guy instance app, which cn run a gtag inside """
+    """ The main guy instance app, which can run a gtag inside """
 
     def __init__(self,gtag,isMultipleSessionPossible=False):
         assert isinstance(gtag,GTag)
@@ -524,7 +619,6 @@ class GTagApp(guy.Guy):
             if(!sessionStorage["gtag"]) sessionStorage["gtag"]=Math.random().toString(36).substring(2);
             var GID=sessionStorage["gtag"];
             async function getSessionId() {return GID}
-            async function render(js) {eval(js)}
         </script>
         %s
     </head>
@@ -549,35 +643,17 @@ class GTagApp(guy.Guy):
         log(">>>SERVE",repr(gtag))
         log(gtag._tree())
 
-        #=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- call statement (old autostart system)
-        caller=""
-        if gtag._call:  # there is an event to call at start !
-            if isAsyncGenerator(gtag._call) or asyncio.iscoroutine(gtag._call):
-                fname,args="",{}
+        await self.js.eval( gtag.bind._start()+";"+gtag._getScripts() )
 
-                if asyncio.iscoroutine(gtag._call):
-                    fname = gtag._call.__name__
-                    args = gtag._call.cr_frame.f_locals  # dict object
-                elif isAsyncGenerator(gtag._call):
-                    fname=gtag._call.__name__
-                    args = gtag._call.ag_frame.f_locals  # dict object
 
-                if "self" in args: del args["self"]
-                args=args.values()  #TODO: how can it works ?! (verify that with TU *a,**k)!)
-
-                method=getattr(gtag.bind,fname)
-                caller=method(*args)
-        #=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
-        js="document.body.innerHTML=`%s`;%s;%s" %(str(gtag),gtag._getScripts(),caller)
-        await self.js.render(js)
-
-    async def forceUpdate(self,g): #can't be called from client side !
-        g._rebuild(False)
-        log(">>>Force UPDATE:",repr(g))
-        await self.js.render( g._update()["script"] )
-
-    async def bindUpdate(self,id:str,gid:str,method:str,args,kargs):
+    async def bindUpdate(self,id:str,gid:str,method:str,args,kargs,jsArgs={}):
         """ inner (js exposed) guy method, called by gtag.bind.<method>(*args) """
+
+        async def forceUpdate(g):
+            g._rebuild(clearScripts=False)
+            log(">>>Force UPDATE:",repr(g))
+            await self.js.eval( g._update()["script"] )
+
         if self._ses is None:
             gtag=self._originalGTag
         else:
@@ -589,11 +665,22 @@ class GTagApp(guy.Guy):
         log("BINDUPDATE on",repr(gtag),"----->",repr(obj),"%s(%s %s)"% (method,args,kargs))
         proc=getattr(obj,method)
 
+        #/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\
+        # dispatch jsArgs in gtag childs
+        #/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\
+        for o in gtag._getChilds().values():
+            defs=jsArgs.get( o.__class__.__name__,{})
+            if defs:
+                for jsk in o._declaredJsInputs.keys():
+                    if jsk in defs:
+                        setattr(o,jsk,defs[jsk])
+        #/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\
+
         if Capacity(proc).has(render.local):
             toRender=obj
         elif Capacity(proc).has(render.parent):
             toRender=obj._parent if obj._parent else obj
-        elif Capacity(proc).has(render.no):
+        elif Capacity(proc).has(render.none):
             toRender=None
         else:
             toRender=gtag
@@ -608,14 +695,14 @@ class GTagApp(guy.Guy):
         if rep:
             if isAsyncGenerator(rep):
                 async for _ in rep: # could use yielded thing to update all or local ?!
-                    assert _ is None, "wtf?"
+                    assert _ is None, "wtf (event returns something)?"
                     if toRender:
-                        await self.forceUpdate(toRender)
+                        await forceUpdate(toRender)
             else:
-                raise Exception("wtf?")
+                raise Exception("wtf (event returns something)?")
 
         if toRender:
-            toRender._rebuild(False)
-            return toRender._update() #UPDATE ALL (historic way)
+            toRender._rebuild(clearScripts=False)
+            return toRender._update()
         #////////////////////////////////////////////////////////////////// THE MAGIC
 
